"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("rxjs/add/observable/of");
var Rx_1 = require("rxjs/Rx");
var manup_service_1 = require("./manup.service");
var MockAppVersion = (function () {
    function MockAppVersion() {
    }
    MockAppVersion.getVersionNumber = function () {
        return Promise.resolve('2.3.4');
    };
    return MockAppVersion;
}());
describe('Manup Spec', function () {
    describe('constructor', function () {
        it('Should configure translation files if it has to', function () {
            var mockTranslate = {
                setTranslation: function () { }
            };
            var config = {
                externalTranslations: false
            };
            spyOn(mockTranslate, 'setTranslation');
            var manup = new manup_service_1.ManUpService(config, null, null, null, null, null, mockTranslate, null);
            expect(mockTranslate.setTranslation).toHaveBeenCalled();
        });
        it('Should not configure translation files using external ones', function () {
            var mockTranslate = {
                setTranslation: function () { }
            };
            var config = {
                externalTranslations: true
            };
            spyOn(mockTranslate, 'setTranslation');
            var manup = new manup_service_1.ManUpService(config, null, null, null, null, null, mockTranslate, null);
            expect(mockTranslate.setTranslation).not.toHaveBeenCalled();
        });
    });
    describe('evaluate', function () {
        it('Should return maintenance mode if json says disabled', function (done) {
            var json = {
                minimum: '2.3.4',
                latest: '2.3.4',
                url: 'http://example.com',
                enabled: false
            };
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, MockAppVersion, null, null);
            manup.evaluate(json).then(function (alert) {
                expect(alert).toEqual(manup_service_1.AlertType.MAINTENANCE);
                done();
            });
        });
        it('Should return mandatory update if app version less than minimum', function (done) {
            var json = {
                minimum: '4.3.4',
                latest: '6.3.4',
                url: 'http://example.com',
                enabled: true
            };
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, MockAppVersion, null, null);
            manup.evaluate(json).then(function (alert) {
                expect(alert).toEqual(manup_service_1.AlertType.MANDATORY);
                done();
            });
        });
        it('Should return optional update if app version less than latest', function (done) {
            var json = {
                minimum: '2.3.4',
                latest: '6.3.4',
                url: 'http://example.com',
                enabled: true
            };
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, MockAppVersion, null, null);
            manup.evaluate(json).then(function (alert) {
                expect(alert).toEqual(manup_service_1.AlertType.OPTIONAL);
                done();
            });
        });
        it('Should return nop if app version latest', function (done) {
            var json = {
                minimum: '2.3.4',
                latest: '2.3.4',
                url: 'http://example.com',
                enabled: true
            };
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, MockAppVersion, null, null);
            manup.evaluate(json).then(function (alert) {
                expect(alert).toEqual(manup_service_1.AlertType.NOP);
                done();
            });
        });
    });
    describe('metadata', function () {
        var config = {
            url: 'test.example.com'
        };
        describe('Http route, no storage configured', function () {
            var mockHttp = {
                get: function (url) {
                    return Rx_1.Observable.of({
                        json: function () {
                            return {
                                ios: {
                                    minimum: '1.0.0',
                                    latest: '2.4.5',
                                    enabled: true,
                                    url: 'http://http.example.com'
                                }
                            };
                        }
                    });
                }
            };
            it('Should make an http request', function (done) {
                spyOn(mockHttp, 'get').and.callThrough();
                var manup = new manup_service_1.ManUpService(config, mockHttp, null, null, null, null, null, null);
                manup.metadata().then(function (data) {
                    expect(mockHttp.get).toHaveBeenCalled();
                    done();
                });
            });
            it('Should return json', function (done) {
                var manup = new manup_service_1.ManUpService(config, mockHttp, null, null, null, null, null, null);
                manup.metadata().then(function (data) {
                    expect(data.ios).toBeDefined();
                    expect(data.ios.url).toBe('http://http.example.com');
                    done();
                });
            });
            it('Should throw an exception if http request fails', function (done) {
                var mockHttpErr = {
                    get: function (url) {
                        return Rx_1.Observable.throw(new Error('no good son'));
                    }
                };
                var manup = new manup_service_1.ManUpService(config, mockHttpErr, null, null, null, null, null, null);
                manup.metadata().then(function (data) {
                    expect(true).toBe(null);
                    done();
                }, function (err) {
                    expect(err).toBeDefined();
                    done();
                });
            });
            it('Should throw an exception if http returns null as it does from time to time', function (done) {
                var mockHttpErr = {
                    get: function (url) {
                        return Rx_1.Observable.of(null);
                    }
                };
                var manup = new manup_service_1.ManUpService(config, mockHttpErr, null, null, null, null, null, null);
                manup.metadata().then(function (data) {
                    expect(true).toBe(null);
                    done();
                }, function (err) {
                    expect(err).toBeDefined();
                    done();
                });
            });
        });
        describe('Http route, with storage', function () {
            var mockHttp = {
                get: function (url) {
                    return Rx_1.Observable.of({
                        json: function () {
                            return {
                                ios: {
                                    minimum: '1.0.0',
                                    latest: '2.4.5',
                                    enabled: true,
                                    url: 'http://http.example.com'
                                }
                            };
                        }
                    });
                }
            };
            var mockStorage = {
                set: function (key, value) {
                    return Promise.resolve();
                }
            };
            var response;
            var manup;
            beforeAll(function (done) {
                manup = new manup_service_1.ManUpService(config, mockHttp, null, null, null, null, null, mockStorage);
                spyOn(mockHttp, 'get').and.callThrough();
                spyOn(manup, 'saveMetadata').and.callThrough();
                manup.metadata().then(function (data) {
                    response = data;
                    done();
                });
            });
            it('Should make an http request', function () {
                expect(mockHttp.get).toHaveBeenCalled();
            });
            it('Should store the result to storage', function () {
                expect(manup.saveMetadata).toHaveBeenCalled();
            });
            it('Should return json', function () {
                expect(response.ios).toBeDefined();
                expect(response.ios.url).toBe('http://http.example.com');
            });
        });
        describe('Fallback to storage', function () {
            var mockHttp;
            var mockStorage;
            beforeEach(function () {
                mockHttp = {
                    get: function (url) {
                        return Rx_1.Observable.throw(new Error('HTTP Failed'));
                    }
                };
                mockStorage = {
                    get: function (key) {
                        return Promise.resolve(JSON.stringify({
                            ios: {
                                minimum: '1.0.0',
                                latest: '2.4.5',
                                enabled: true,
                                url: 'http://storage.example.com'
                            }
                        }));
                    },
                    set: function (key, value) {
                        return Promise.resolve();
                    }
                };
                spyOn(mockHttp, 'get').and.callThrough();
                spyOn(mockStorage, 'get').and.callThrough();
                spyOn(mockStorage, 'set').and.callThrough();
            });
            it('Should make an http request', function (done) {
                var manup = new manup_service_1.ManUpService(config, mockHttp, null, null, null, null, null, mockStorage);
                manup.metadata().then(function (data) {
                    expect(mockHttp.get).toHaveBeenCalled();
                    done();
                });
            });
            it('Should fallback to storage', function (done) {
                var manup = new manup_service_1.ManUpService(config, mockHttp, null, null, null, null, null, mockStorage);
                manup.metadata().then(function (data) {
                    expect(mockStorage.get).toHaveBeenCalled();
                    done();
                });
            });
            it('Should return json', function (done) {
                var manup = new manup_service_1.ManUpService(config, mockHttp, null, null, null, null, null, mockStorage);
                manup.metadata().then(function (data) {
                    expect(data.ios).toBeDefined();
                    expect(data.ios.url).toBe('http://storage.example.com');
                    done();
                });
            });
        });
    });
    describe('metadataFromStorage', function () {
        it('Should return data from storage, if it exists', function (done) {
            var metadata = {
                ios: { minimum: '1.0.0', latest: '2.0.0', enabled: true, url: 'test.example.com' }
            };
            var mockStorage = {
                get: function (key) {
                    return Promise.resolve(JSON.stringify(metadata));
                }
            };
            spyOn(mockStorage, 'get').and.callThrough();
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, null, null, mockStorage);
            manup.metadataFromStorage().then(function (data) {
                expect(mockStorage.get).toHaveBeenCalledWith('com.nextfaze.ionic-manup.manup');
                expect(data).toEqual(metadata);
                done();
            });
        });
        it('Should error if data is not stored ', function (done) {
            var mockStorage = {
                get: function (key) {
                    return Promise.reject(new Error('not found'));
                }
            };
            spyOn(mockStorage, 'get').and.callThrough();
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, null, null, mockStorage);
            manup.metadataFromStorage().then(function (data) {
                expect(true).toBe(false);
            }, function (error) {
                expect(error).toBeDefined();
                done();
            });
        });
        it('Should throw an exception if storage not configured', function (done) {
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, null, null, null);
            expect(function () {
                manup.metadataFromStorage();
            }).toThrowError();
            done();
        });
    });
    describe('saveMetaData', function () {
        var mockStorage = {
            set: function (key, value) {
                return Promise.resolve();
            }
        };
        it('Should save the item if storage configured', function (done) {
            spyOn(mockStorage, 'set').and.callThrough();
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, null, null, mockStorage);
            var metadata = {
                ios: { minimum: '1.0.0', latest: '2.0.0', enabled: true, url: 'test.example.com' }
            };
            manup.saveMetadata(metadata).then(function () {
                expect(mockStorage.set).toHaveBeenCalledWith('com.nextfaze.ionic-manup.manup', JSON.stringify(metadata));
                done();
            });
        });
        it('Should throw an exception if storage not configured', function (done) {
            var metadata = {
                ios: { minimum: '1.0.0', latest: '2.0.0', enabled: true, url: 'test.example.com' }
            };
            var manup = new manup_service_1.ManUpService(null, null, null, null, null, null, null, null);
            expect(function () {
                manup.saveMetadata(metadata);
            }).toThrowError();
            done();
        });
    });
    describe('getPlatformData', function () {
        var json = {
            ios: {
                minimum: '1.0.0',
                latest: '2.4.5',
                enabled: true,
                url: 'http://example.com'
            },
            android: {
                minimum: '4.0.1',
                latest: '6.2.1',
                enabled: true,
                url: 'http://example.com'
            },
            windows: {
                minimum: '1.0.0',
                latest: '1.0.1',
                enabled: false,
                url: 'http://example.com'
            }
        };
        it('should return IOS metadata if platform is ios', function () {
            var mockPlatform = {
                is: function (v) {
                    return v === 'ios';
                }
            };
            var manup = new manup_service_1.ManUpService(null, null, null, mockPlatform, null, null, null, null);
            var result = manup.getPlatformData(json);
            expect(result).toEqual(json.ios);
        });
        it('should return android metadata if platform is android', function () {
            var mockPlatform = {
                is: function (v) {
                    return v === 'android';
                }
            };
            var manup = new manup_service_1.ManUpService(null, null, null, mockPlatform, null, null, null, null);
            var result = manup.getPlatformData(json);
            expect(result).toEqual(json.android);
        });
        it('should return windows metadata if platform is windows', function () {
            var mockPlatform = {
                is: function (v) {
                    return v === 'windows';
                }
            };
            var manup = new manup_service_1.ManUpService(null, null, null, mockPlatform, null, null, null, null);
            var result = manup.getPlatformData(json);
            expect(result).toEqual(json.windows);
        });
        it('should throw and error if the platform is unsupported', function () {
            var mockPlatform = {
                is: function (v) {
                    return false;
                }
            };
            var manup = new manup_service_1.ManUpService(null, null, null, mockPlatform, null, null, null, null);
            expect(function () {
                manup.getPlatformData(json);
            }).toThrow();
        });
        it('should throw and error invalid metadata is passed in', function () {
            var mockPlatform = {
                is: function (v) {
                    return false;
                }
            };
            var manup = new manup_service_1.ManUpService(null, null, null, mockPlatform, null, null, null, null);
            expect(function () {
                manup.getPlatformData(null);
            }).toThrow();
        });
    });
});
//# sourceMappingURL=manup.spec.js.map